const {registerPlugin} = wp.plugins;
const {PluginSidebar, PluginSidebarMoreMenuItem} = wp.editPost;
const {TextControl, PanelBody, PanelRow} = wp.components;
const {BlockControls,InspectorControls} = wp.editor;

const yasrDisableAutoInsert = __('Disable auto insert for this post or page?', 'yet-another-stars-rating');
const yasrSelectSnippetTitle = __('This review is about a... ', 'yet-another-stars-rating');
const yasrSetItemTypeProduct = __('Product', 'yet-another-stars-rating');
const yasrSetItemTypePlace = __('Place', 'yet-another-stars-rating');
const yasrSetItemTypeRecipe = __('Recipe', 'yet-another-stars-rating');
const yasrSetItemTypeOther = __('Other', 'yet-another-stars-rating');


class YasrSideBarAutoInsert extends React.Component {

    constructor(props) {
        super(props);

        let isThisPostExluded = wp.data.select('core/editor').getCurrentPost().meta.yasr_auto_insert_disabled;
        let isThisPostExludedCheckbox = false;

        if (isThisPostExluded === 'yes') {
            isThisPostExludedCheckbox = true;
        }

        this.state = {postExcluded: isThisPostExludedCheckbox};

        this.yasrUpdatePostMetaAutoInsert = this.yasrUpdatePostMetaAutoInsert.bind(this);
    }

     yasrUpdatePostMetaAutoInsert(event) {

        const target = event.target;
        const postExcluded = target.type === 'checkbox' ? target.checked : target.value;

        this.setState({postExcluded: postExcluded});

        if (postExcluded === true) {
            wp.data.dispatch('core/editor').editPost(
                { meta: { yasr_auto_insert_disabled: 'yes' } }
            );
        } else {
            wp.data.dispatch('core/editor').editPost(
                { meta: { yasr_auto_insert_disabled: 'no' } }
            );
        }

    }

    render () {
        return (
            <div className="yasr-guten-block-panel-center">
                <label><span>{yasrDisableAutoInsert}</span></label>
                <div className="yasr-onoffswitch-big" id="yasr-switcher-disable-auto-insert">
                    <input type="checkbox"
                           name="yasr_auto_insert_disabled"
                           className="yasr-onoffswitch-checkbox"
                           value="yes"
                           id="yasr-auto-insert-disabled-switch"
                           defaultChecked={this.state.postExcluded}
                           onChange={this.yasrUpdatePostMetaAutoInsert}
                    />
                    <label className="yasr-onoffswitch-label" htmlFor="yasr-auto-insert-disabled-switch">
                        <span className="yasr-onoffswitch-inner"></span>
                        <span className="yasr-onoffswitch-switch"></span>
                    </label>
                </div>
                <hr/>
            </div>
        );
    }

}

class YasrSideBarSelectSnippets extends React.Component {

    constructor(props) {
        super(props);

        //at the begin, get the default type
        let snippetType = yasrConstantGutenberg.defaultItemType;

        //postmeta itemtype
        let postMetaItemType = wp.data.select('core/editor').getCurrentPost().meta.yasr_review_type;

        //if post meta value is one of this overwrite snippetType
        if (postMetaItemType === 'Product' || postMetaItemType === 'Place' || postMetaItemType === 'Recipe'
            || postMetaItemType === 'Other') {
            snippetType = postMetaItemType;
        }

        this.state = {snippet: snippetType};

        this.yasrSetItemType = this.yasrSetItemType.bind(this);
    }

    yasrSetItemType(event) {
        this.setState({snippet: event.target.value});

        wp.data.dispatch( 'core/editor' ).editPost(
            { meta: { yasr_review_type: event.target.value } }
        );
    }

    render() {
        return (
            <div className="yasr-guten-block-panel-center">
                <label><span>{yasrSelectSnippetTitle}</span></label>
                <form>
                    <select value={this.state.snippet} onChange={this.yasrSetItemType}>
                        <option value="Product">{yasrSetItemTypeProduct}</option>
                        <option value="Place">{yasrSetItemTypePlace}</option>
                        <option value="Recipe">{yasrSetItemTypeRecipe}</option>
                        <option value="Other">{yasrSetItemTypeOther}</option>
                    </select>
                </form>
            </div>
        );
    }
}

class yasrSidebar extends React.Component {

    constructor(props) {
        super(props);

        let yasrAutoInsertEnabled = false;

        //this is not for the post, but from settings
        if (yasrConstantGutenberg.autoInsert !== 'disabled') {
            yasrAutoInsertEnabled = true;
        }

        this.state = {yasrAutoInsertEnabled: yasrAutoInsertEnabled};

    }

    render() {
        return (
            <Fragment>
                <PluginSidebarMoreMenuItem name="my-plugin-sidebar"  type="sidebar" target="yasr-guten-sidebar" >
                    { __( 'YASR post settings', 'yet-another-stars-rating' ) }
                </PluginSidebarMoreMenuItem>
                <PluginSidebar name="yasr-guten-sidebar" title="YASR Settings">
                    <PanelBody>
                        {this.state.yasrAutoInsertEnabled && <YasrSideBarAutoInsert/>}
                        <YasrSideBarSelectSnippets/>
                    </PanelBody>
                </PluginSidebar>
            </Fragment>
        );
    }
}


// Our custom sidebar
registerPlugin( 'yasr-sidebar', {
    icon: 'star-half',
    title: __( 'Yasr: Page Settings', 'yet-another-stars-rating' ),
    render: yasrSidebar
} );